@@ -1,20 +1,6 @@
 /*
- * Copyright (C) 1995-2010 University of Karlsruhe.  All right reserved.
- *
  * This file is part of libFirm.
- *
- * This file may be distributed and/or modified under the terms of the
- * GNU General Public License version 2 as published by the Free Software
- * Foundation and appearing in the file LICENSE.GPL included in the
- * packaging of this file.
- *
- * Licensees holding valid libFirm Professional Edition licenses may use
- * this file in accordance with the libFirm Commercial License.
- * Agreement provided with the Software.
- *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE.
+ * Copyright (C) 2012 University of Karlsruhe.
  */
 
 /**
@@ -63,29 +49,27 @@
  *    Implementation of the FIRM operations: ir_node
  *    ----------------------------------------------
  *
- *      Ir_nodes represent operations on the data of the program and control flow
- *      operations.  Examples of ir_nodes:  Add, Jmp, Cmp
+ *      Ir_nodes represent operations on the data of the program and control
+ *      flow operations.  Examples of ir_nodes:  Add, Jmp, Cmp
  *
- *      FIRM is a dataflow graph.  A dataflow graph is a directed graph,
- *      so that every node has incoming and outgoing edges.  A node is
- *      executable if every input at its incoming edges is available.
- *      Execution of the dataflow graph is started at the Start node which
- *      has no incoming edges and ends when the End node executes, even if
- *      there are still executable or not executed nodes.  (Is this true,
- *      or must all executable nodes be executed?)  (There are exceptions
- *      to the dataflow paradigma that all inputs have to be available
- *      before a node can execute: Phi, Block.  See UKA Techreport
- *      1999-14.)
+ *      FIRM is a dataflow graph.  A dataflow graph is a directed graph, so that
+ *      every node has incoming and outgoing edges.  A node is executable if
+ *      every input at its incoming edges is available.  Execution of the
+ *      dataflow graph is started at the Start node which has no incoming edges
+ *      and ends when the End node executes, even if there are still executable
+ *      or not executed nodes.  (Is this true, or must all executable nodes be
+ *      executed?)  (There are exceptions to the dataflow paradigma that all
+ *      inputs have to be available before a node can execute: Phi, Block.  See
+ *      UKA Techreport 1999-14.)
  *
- *      The implementation of FIRM differs from the view as a dataflow
- *      graph.  To allow fast traversion of the graph edges are
- *      implemented as C-pointers.  Inputs to nodes are not ambiguous, the
- *      results can be used by several other nodes.  Each input can be
- *      implemented as a single pointer to a predecessor node, outputs
- *      need to be lists of pointers to successors.  Therefore a node
- *      contains pointers to its predecessors so that the implementation is a
- *      dataflow graph with reversed edges.  It has to be traversed bottom
- *      up.
+ *      The implementation of FIRM differs from the view as a dataflow graph.
+ *      To allow fast traversion of the graph edges are implemented as
+ *      C-pointers.  Inputs to nodes are not ambiguous, the results can be used
+ *      by several other nodes.  Each input can be implemented as a single
+ *      pointer to a predecessor node, outputs need to be lists of pointers to
+ *      successors.  Therefore a node contains pointers to its predecessors so
+ *      that the implementation is a dataflow graph with reversed edges.  It has
+ *      to be traversed bottom up.
  *
  *      All nodes of the IR have the same basic structure.  They are
  *      distinguished by a field containing the opcode.
@@ -161,28 +145,28 @@
  *      for each new node.
  *    - An even less comfortable interface where the block needs to be specified
  *      explicitly.  This is called the "raw" interface. (new_r_<Node>
- *      constructors).  These nodes are not optimized.
+ *      constructors).
  *
- *    To use the functionality of the comfortable interface correctly the Front
- *    End needs to follow certain protocols.  This is explained in the following.
- *    To build a correct IR with the other interfaces study the semantics of
- *    the firm node (See tech-reprot UKA 1999-14).  For the construction of
- *    types and entities see the documentation in those modules.
+ *    To use the functionality of the comfortable interface correctly the front
+ *    end needs to follow certain protocols.  This is explained in the
+ *    following.  To build a correct IR with the other interfaces study the
+ *    semantics of the firm node (See tech-reprot UKA 1999-14).  For the
+ *    construction of types and entities see the documentation in these modules.
  *
- *    First the Frontend needs to decide which variables and values used in
- *    a procedure can be represented by dataflow edges.  These are variables
- *    that need not be saved to memory as they cause no side effects visible
- *    out of the procedure.  Often these are all compiler generated
- *    variables and simple local variables of the procedure as integers,
- *    reals and pointers.  The frontend has to count and number these variables.
+ *    First the front end needs to decide which variables and values used in a
+ *    procedure can be represented by dataflow edges.  These are variables that
+ *    need not be saved to memory as they cause no side effects visible out of
+ *    the procedure.  Often these are all compiler generated variables and
+ *    simple local variables of the procedure as integers, reals and pointers.
+ *    The front end has to count and number these variables.
  *
  *    First an ir_graph needs to be constructed with new_ir_graph.  The
  *    constructor gets the number of local variables.  The graph is held in the
  *    global variable irg.
  *
  *    Now the construction of the procedure can start.  Several basic blocks can
- *    be constructed in parallel, but the code within each block needs to
- *    be constructed (almost) in program order.
+ *    be constructed in parallel, but the code within each block needs to be
+ *    constructed (almost) in program order.
  *
  *    A global variable holds the current basic block.  All (non block) nodes
  *    generated are added to this block.  The current block can be set with
@@ -190,26 +174,27 @@
  *    switches need to be performed constantly.
  *
  *    To generate a Block node (with the comfortable interface), its predecessor
- *    control flow nodes need not be known.  In case of cyclic control flow these
- *    can not be known when the block is constructed.  With add_immBlock_pred(block,
- *    cfnode) predecessors can be added to the block.  If all predecessors are
- *    added to the block mature_immBlock(b) needs to be called.  Calling mature_immBlock
- *    early improves the efficiency of the Phi node construction algorithm.
- *    But if several  blocks are constructed at once, mature_immBlock must only
- *    be called after performing all set_values and set_stores in the block!
- *    (See documentation of new_immBlock constructor.)
+ *    control flow nodes need not be known.  In case of cyclic control flow
+ *    these can not be known when the block is constructed.  With
+ *    add_immBlock_pred(block, cfnode) predecessors can be added to the block.
+ *    If all predecessors are added to the block mature_immBlock(b) needs to be
+ *    called.  Calling mature_immBlock early improves the efficiency of the Phi
+ *    node construction algorithm.  But if several  blocks are constructed at
+ *    once, mature_immBlock must only be called after performing all set_values
+ *    and set_stores in the block!  (See documentation of new_immBlock
+ *    constructor.)
  *
- *    The constructors of arithmetic nodes require that their predecessors
- *    are mentioned.  Sometimes these are available in the Frontend as the
- *    predecessors have just been generated by the frontend.  If they are local
+ *    The constructors of arithmetic nodes require that their predecessors are
+ *    mentioned.  Sometimes these are available in the Frontend as the
+ *    predecessors have just been generated by the front end.  If they are local
  *    values, the predecessors can be obtained from the library with a call to
- *    get_value(local_val_nr).  (local_val_nr needs to be administered by
- *    the Frontend.)  A call to get_value triggers the generation of Phi nodes.
- *    If an arithmetic operation produces a local value, this value needs to be
+ *    get_value(local_val_nr).  (local_val_nr needs to be administered by the
+ *    Frontend.)  A call to get_value triggers the generation of Phi nodes.  If
+ *    an arithmetic operation produces a local value, this value needs to be
  *    passed to the library by set_value(node, local_val_nr).
  *    In straight line code these two operations just remember and return the
- *    pointer to nodes producing the value.  If the value passes block boundaries
- *    Phi nodes can be inserted.
+ *    pointer to nodes producing the value.  If the value passes block
+ *    boundaries Phi nodes can be inserted.
  *    Similar routines exist to manage the Memory operands: set_store and
  *    get_store.
  *
@@ -218,43 +203,46 @@
  *    extracted by proj nodes.
  *
  *    The following example illustrates the construction of a simple basic block
- *    with two predecessors stored in variables cf_pred1 and cf_pred2, containing
- *    the code
+ *    with two predecessors stored in variables cf_pred1 and cf_pred2,
+ *    containing the code
  *      a = a div a;
  *    and finally jumping to an other block.  The variable a got the local_val_nr
- *    42 by the frontend.
+ *    42 by the front end.
  *
- *    ir_node *this_block, *cf_pred1, *cf_pred2, *a_val, *mem, *div, *res, *cf_op;
- *
- *    this_block = new_immBlock();
- *    add_immBlock_pred(this_block, cf_pred1);
- *    add_immBlock_pred(this_block, cf_pred2);
- *    mature_immBlock(this_block);
- *    a_val = get_value(42, mode_Iu);
- *    mem = get_store();
- *    div = new_Div(mem, a_val, a_val, mode_Iu);
- *    mem = new_Proj(div, mode_M, pn_Div_M);   * for the numbers for Proj see docu *
- *    res = new_Proj(div, mode_Iu, pn_Div_res);
- *    set_store(mem);
- *    set_value(res, 42);
- *    cf_op = new_Jmp();
+ *    ir_node *example(ir_node *cf_pred1, ir_node *cf_pred2)
+ *    {
+ *      ir_node *this_block = new_immBlock();
+ *      add_immBlock_pred(this_block, cf_pred1);
+ *      add_immBlock_pred(this_block, cf_pred2);
+ *      mature_immBlock(this_block);
+ *      set_cur_block(this_block);
+ *      ir_node *a_val = get_value(42, mode_Iu);
+ *      ir_node *div   = new_Div(get_store(), a_val, a_val, mode_Iu);
+ *      ir_node *mem   = new_Proj(div, mode_M, pn_Div_M);
+ *      ir_node *res   = new_Proj(div, mode_Iu, pn_Div_res);
+ *      set_store(mem);
+ *      set_value(res, 42);
+ *      ir_node *cf_op = new_Jmp();
+ *      return cf_op;
+ *    }
  *
  *    For further information look at the documentation of the nodes and
- *    constructors and at the paragraph COPING WITH DATA OBJECTS at the
- *    end of this documentation.
+ *    constructors and at the paragraph COPING WITH DATA OBJECTS at the end of
+ *    this documentation.
  *
  *    IR_NODES AND CONSTRUCTORS FOR IR_NODES
  *    =======================================
  *
- *    All ir_nodes are defined by a common data structure.  They are distinguished
- *    by their opcode and differ in the number of their attributes.
+ *    All ir_nodes are defined by a common data structure.  They are
+ *    distinguished by their opcode and differ in the number of their
+ *    attributes.
  *
- *    Const nodes are always added to the start block.
- *    All other constructors add the created node to the current_block.
- *    swich_block(block) allows to set the current block to block.
+ *    Const nodes are always added to the start block.  All other constructors
+ *    add the created node to the current_block.  set_cur_block(block) allows to
+ *    set the current block to block.
  *
- *    Watch for my inconsistent use of input and predecessor (dataflow view)
- *    and `the node points to' (implementation view).
+ *    Watch for my inconsistent use of input and predecessor (dataflow view) and
+ *    `the node points to' (implementation view).
  *
  *    The following description of the nodes lists four properties them if these
  *    are of interest:
@@ -268,41 +256,38 @@
  *    COPING WITH DATA OBJECTS
  *    ========================
  *
- *    Two kinds of data objects have to be distinguished for generating
- *    FIRM.  First there are local variables other than arrays that are
- *    known to be alias free.  Second there are all other data objects.
- *    For the first a common SSA representation is built, the second
- *    are modeled by saving them to memory.  The memory is treated as
- *    a single local variable, the alias problem is hidden in the
- *    content of this variable.
+ *    Two kinds of data objects have to be distinguished for generating FIRM.
+ *    First there are local variables other than arrays that are known to be
+ *    alias free.  Second there are all other data objects.  For the first a
+ *    common SSA representation is built, the second are modeled by saving them
+ *    to memory.  The memory is treated as a single local variable, the alias
+ *    problem is hidden in the content of this variable.
  *
- *    All values known in a Block are listed in the block's attribute,
- *    block.**graph_arr which is used to automatically insert Phi nodes.
- *    The following two functions can be used to add a newly computed value
- *    to the array, or to get the producer of a value, i.e., the current
- *    live value.
+ *    All values known in a Block are listed in the block's attribute
+ *    block.graph_arr which is used to automatically insert Phi nodes.  The
+ *    following two functions can be used to add a newly computed value to the
+ *    array, or to get the producer of a value, i.e., the current live value.
  *
- *    inline void set_value (int pos, ir_node *value)
+ *    void set_value(int pos, ir_node *value)
  *    -----------------------------------------------
  *
- *    Has to be called for every assignment to a local variable.  It
- *    adds the value to the array of used values at position pos.  Pos
- *    has to be a unique identifier for an entry in the procedure's
- *    definition table.  It can be used to access the value again.
- *    Requires current_block to be set correctly.
+ *    Has to be called for every assignment to a local variable.  It adds the
+ *    value to the array of used values at position pos.  Pos has to be a unique
+ *    identifier for an entry in the procedure's definition table.  It can be
+ *    used to access the value again.  Requires current_block to be set
+ *    correctly.
  *
- *    ir_node *get_value (int pos, ir_mode *mode)
+ *    ir_node *get_value(int pos, ir_mode *mode)
  *    -------------------------------------------
  *
- *    Returns the node defining the value referred to by pos. If the
- *    value is not defined in this block a Phi node is generated and
- *    all definitions reaching this Phi node are collected.  It can
- *    happen that the algorithm allocates an unnecessary Phi node,
- *    e.g. if there is only one definition of this value, but this
- *    definition reaches the currend block on several different
- *    paths.  This Phi node will be eliminated if optimizations are
- *    turned on right after its creation.
- *    Requires current_block to be set correctly.
+ *    Returns the node defining the value referred to by pos. If the value is
+ *    not defined in this block a Phi node is generated and all definitions
+ *    reaching this Phi node are collected.  It can happen that the algorithm
+ *    allocates an unnecessary Phi node, e.g. if there is only one definition of
+ *    this value, but this definition reaches the currend block on several
+ *    different paths.  This Phi node will be eliminated if optimizations are
+ *    turned on right after its creation.  Requires current_block to be set
+ *    correctly.
  *
  *    There are two special routines for the global store:
  */
@@ -322,9 +307,7 @@
  *
  * Adds the node to the start block.
  *
- * Constructor for a Const node. The constant represents a target
- * value.  Sets the type information to type_unknown.  (No more
- * supported: If tv is entity derives a somehow useful type.)
+ * The constant represents a target value.
  *
  * @param *db    A pointer for debug information.
  * @param *irg   The IR graph the node  belongs to.
@@ -338,9 +321,7 @@ FIRM_API ir_node *new_rd_Const_long(dbg_info *db, ir_graph *irg,
  *
  * Adds the node to the start block.
  *
- * Constructor for a Const node. The constant represents a target
- * value.  Sets the type information to type_unknown.  (No more
- * supported: If tv is entity derives a somehow useful type.)
+ * The constant represents a target value.
  *
  * @param *irg   The IR graph the node  belongs to.
  * @param *mode  The mode of the operands and the results.
@@ -372,257 +353,22 @@ FIRM_API ir_node *new_Const_long(ir_mode *mode, long value);
 
 /** @} */
 
-/** addtogroup SymConst
+/** @addtogroup Phi
  * @{
  */
 
-/** Constructor for a SymConst node.
- *
- *  This is the constructor for a symbolic constant.
- *    There are several kinds of symbolic constants:
- *    - symconst_type_size  The symbolic constant represents the size of a type.
- *                          The type of which the constant represents the size
- *                          is given explicitly.
- *    - symconst_type_align The symbolic constant represents the alignment of a
- *                          type.  The type of which the constant represents the
- *                          size is given explicitly.
- *    - symconst_addr_ent   The symbolic constant represents the address of an
- *                          entity (variable or method).  The variable is given
- *                          explicitly by a firm entity.
- *    - symconst_ofs_ent    The symbolic constant represents the offset of an
- *                          entity in its owner type.
- *    - symconst_enum_const The symbolic constant is a enumeration constant of
- *                          an enumeration type.
- *
- *    Inputs to the node:
- *      No inputs except the block it belongs to.
- *    Outputs of the node.
- *      An unsigned integer (I_u) or a pointer (P).
- *
- *    Mention union in declaration so that the firmjni generator recognizes that
- *    it can not cast the argument to an int.
- *
- * @param *db     A pointer for debug information.
- * @param *irg    The IR graph the node  belongs to.
- * @param mode    The mode for the SymConst.
- * @param value   A type, ident, entity or enum constant depending on the
- *                SymConst kind.
- * @param kind    The kind of the symbolic constant, see the list above
- */
-FIRM_API ir_node *new_rd_SymConst(dbg_info *db, ir_graph *irg, ir_mode *mode,
-                                  union symconst_symbol value,
-                                  symconst_kind kind);
+/** Constructor for memory Phi with keep-alive edge. */
+FIRM_API ir_node *new_rd_Phi_loop(dbg_info *db, ir_node *block,
+                                      int arity, ir_node *in[]);
 
-/** Constructor for a SymConst addr_ent node.
- *
- * Same as new_rd_SymConst, except that the constructor is tailored for
- * symconst_addr_ent.
- * Adds the SymConst to the start block of irg.
- */
-FIRM_API ir_node *new_rd_SymConst_addr_ent(dbg_info *db, ir_graph *irg,
-                                           ir_mode *mode, ir_entity *symbol);
+/** Constructor for memory Phi with keep-alive edge. */
+FIRM_API ir_node *new_r_Phi_loop(ir_node *block, int arity, ir_node *in[]);
 
-/** Constructor for a SymConst ofs_ent node.
- *
- * Same as new_rd_SymConst, except that the constructor is tailored for
- * symconst_ofs_ent.
- * Adds the SymConst to the start block of irg.
- */
-FIRM_API ir_node *new_rd_SymConst_ofs_ent(dbg_info *db, ir_graph *irg,
-                                          ir_mode *mode, ir_entity *symbol);
+/** Constructor for memory Phi with keep-alive edge. */
+FIRM_API ir_node *new_d_Phi_loop(dbg_info *db, int arity, ir_node *in[]);
 
-/** Constructor for a SymConst size node.
- *
- * Same as new_rd_SymConst, except that the constructor is tailored for
- * symconst_type_size.
- * Adds the SymConst to the start block of irg.
- */
-FIRM_API ir_node *new_rd_SymConst_size(dbg_info *db, ir_graph *irg,
-                                       ir_mode *mode, ir_type *symbol);
-
-/** Constructor for a SymConst size node.
- *
- * Same as new_rd_SymConst, except that the constructor is tailored for
- * symconst_type_align.
- * Adds the SymConst to the start block of irg.
- */
-FIRM_API ir_node *new_rd_SymConst_align(dbg_info *db, ir_graph *irg,
-                                        ir_mode *mode, ir_type *symbol);
-
-/** Constructor for a SymConst node.
- *
- *  This is the constructor for a symbolic constant.
- *    There are several kinds of symbolic constants:
- *    - symconst_type_size  The symbolic constant represents the size of a type.
- *                          The type of which the constant represents the size
- *                          is given explicitly.
- *    - symconst_type_align The symbolic constant represents the alignment of a
- *                          type.  The type of which the constant represents the
- *                          size is given explicitly.
- *    - symconst_addr_ent   The symbolic constant represents the address of an
- *                          entity (variable or method).  The variable is given
- *                          explicitly by a firm entity.
- *    - symconst_ofs_ent    The symbolic constant represents the offset of an
- *                          entity in its owner type.
- *    - symconst_enum_const The symbolic constant is a enumeration constant of
- *                          an enumeration type.
- *
- *    Inputs to the node:
- *      No inputs except the block it belongs to.
- *    Outputs of the node.
- *      An unsigned integer (I_u) or a pointer (P).
- *
- *    Mention union in declaration so that the firmjni generator recognizes that
- *    it can not cast the argument to an int.
- *
- * @param *irg    The IR graph the node  belongs to.
- * @param mode    The mode for the SymConst.
- * @param value   A type, ident, entity or enum constant depending on the
- *                SymConst kind.
- * @param kind    The kind of the symbolic constant, see the list above
- */
-FIRM_API ir_node *new_r_SymConst(ir_graph *irg, ir_mode *mode,
-                                 union symconst_symbol value,
-                                 symconst_kind kind);
-
-/** Constructor for an SymConst node
- *
- *  This is the constructor for a symbolic constant.
- *    There are several kinds of symbolic constants:
- *    - symconst_type_size  The symbolic constant represents the size of a type.
- *                          The type of which the constant represents the size
- *                          is given explicitly.
- *    - symconst_type_align The symbolic constant represents the alignment of a
- *                          type.  The type of which the constant represents the
- *                          size is given explicitly.
- *    - symconst_addr_ent   The symbolic constant represents the address of an
- *                          entity (variable or method).  The variable is given
- *                          explicitly by a firm entity.
- *    - symconst_ofs_ent    The symbolic constant represents the offset of an
- *                          entity in its owner type.
- *    - symconst_enum_const The symbolic constant is a enumeration constant of
- *                          an enumeration type.
- *
- *    Inputs to the node:
- *      No inputs except the block it belongs to.
- *    Outputs of the node.
- *      An unsigned integer (I_u) or a pointer (P).
- *
- *    Mention union in declaration so that the firmjni generator recognizes that
- *    it can not cast the argument to an int.
- *
- * @param *db     A pointer for debug information.
- * @param mode    The mode for the SymConst.
- * @param value   A type, ident, entity or enum constant depending on the
- *                SymConst kind.
- * @param kind    The kind of the symbolic constant, see the list above
- */
-FIRM_API ir_node *new_d_SymConst(dbg_info *db, ir_mode *mode,
-                                 union symconst_symbol value,
-                                 symconst_kind kind);
-
-/** Constructor for a SymConst node.
- *
- *  This is the constructor for a symbolic constant.
- *    There are several kinds of symbolic constants:
- *    - symconst_type_size  The symbolic constant represents the size of a type.
- *                          The type of which the constant represents the size
- *                          is given explicitly.
- *    - symconst_type_align The symbolic constant represents the alignment of a
- *                          type.  The type of which the constant represents the
- *                          size is given explicitly.
- *    - symconst_addr_ent   The symbolic constant represents the address of an
- *                          entity (variable or method).  The variable is given
- *                          explicitly by a firm entity.
- *    - symconst_ofs_ent    The symbolic constant represents the offset of an
- *                          entity in its owner type.
- *    - symconst_enum_const The symbolic constant is a enumeration constant of
- *                          an enumeration type.
- *
- *    Inputs to the node:
- *      No inputs except the block it belongs to.
- *    Outputs of the node.
- *      An unsigned integer (I_u) or a pointer (P).
- *
- *    Mention union in declaration so that the firmjni generator recognizes that
- *    it can not cast the argument to an int.
- *
- * @param mode    The mode for the SymConst.
- * @param value   A type, ident, entity or enum constant depending on the
- *                SymConst kind.
- * @param kind    The kind of the symbolic constant, see the list above
- */
-FIRM_API ir_node *new_SymConst(ir_mode *mode, union symconst_symbol value,
-                               symconst_kind kind);
-
-/** @} */
-
-/** @addtogroup Sel
- * @{
- */
-
-/** Constructor for a simpleSel node.
- *
- *  This is a shortcut for the new_rd_Sel() constructor.  To be used for
- *  Sel nodes that do not select from an array, i.e., have no index
- *  inputs.  It adds the two parameters 0, NULL.
- *
- * @param   *db        A pointer for debug information.
- * @param   *block     The IR block the node belongs to.
- * @param   *store     The memory in which the object the entity should be
- *                     selected from is allocated.
- * @param   *objptr    The object from that the Sel operation selects a
- *                     single attribute out.
- * @param   *ent       The entity to select.
- */
-FIRM_API ir_node *new_rd_simpleSel(dbg_info *db, ir_node *block, ir_node *store,
-                                   ir_node *objptr, ir_entity *ent);
-
-/** Constructor for a simpleSel node.
- *
- *  This is a shortcut for the new_d_Sel() constructor.  To be used for
- *  Sel nodes that do not select from an array, i.e., have no index
- *  inputs.  It adds the two parameters 0, NULL.
- *
- * @param *block     The IR block the node belongs to.
- * @param *store     The memory in which the object the entity should be selected
- *                   from is allocated.
- * @param *objptr    The object from that the Sel operation selects a
- *                   single attribute out.
- * @param *ent       The entity to select.
- * @ingroup Sel
- */
-FIRM_API ir_node *new_r_simpleSel(ir_node *block, ir_node *store,
-                                  ir_node *objptr, ir_entity *ent);
-
-/** Constructor for a simpleSel node.
- *
- *  This is a shortcut for the new_d_Sel() constructor.  To be used for
- *  Sel nodes that do not select from an array, i.e., have no index
- *  inputs.  It adds the two parameters 0, NULL.
- *
- * @param   *db        A pointer for debug information.
- * @param   *store     The memory in which the object the entity should be
- *                     selected from is allocated.
- * @param   *objptr    The object from that the Sel operation selects a
- *                     single attribute out.
- * @param   *ent       The entity to select.
- */
-FIRM_API ir_node *new_d_simpleSel(dbg_info *db, ir_node *store, ir_node *objptr,
-                                  ir_entity *ent);
-
-/** Constructor for a simpelSel node.
- *
- *  This is a shortcut for the new_Sel() constructor.  To be used for
- *  Sel nodes that do not select from an array, i.e., have no index
- *  inputs.  It adds the two parameters 0, NULL.
- *
- * @param   *store     The memory in which the object the entity should be selected from is allocated.
- * @param   *objptr    The object from that the Sel operation selects a single attribute out.
- * @param   *ent       The entity to select.
- */
-FIRM_API ir_node *new_simpleSel(ir_node *store, ir_node *objptr,
-                                ir_entity *ent);
+/** Constructor for memory Phi with keep-alive edge. */
+FIRM_API ir_node *new_Phi_loop(int arity, ir_node *in[]);
 
 /** @} */
 
@@ -638,11 +384,11 @@ FIRM_API ir_node *new_simpleSel(ir_node *store, ir_node *objptr,
  * @param   *op1   The first operand.
  * @param   *op2   The second operand.
  * @param   *mode  The mode of the result.
- * @param   state  The pinned state.
+ * @param   pinned Whether the node is pinned in its block.
  */
 FIRM_API ir_node *new_rd_DivRL(dbg_info *db, ir_node *block, ir_node *memop,
                                ir_node *op1, ir_node *op2, ir_mode *mode,
-                               op_pin_state state);
+                               int pinned);
 
 /** Constructor for a remainderless Div node.
  *
@@ -651,11 +397,11 @@ FIRM_API ir_node *new_rd_DivRL(dbg_info *db, ir_node *block, ir_node *memop,
  * @param *op1   The first operand.
  * @param *op2   The second operand.
  * @param *mode  The mode of the result.
- * @param state  The pinned state.
+ * @param pinned Whether the node is pinned in its block.
  */
 FIRM_API ir_node *new_r_DivRL(ir_node *block, ir_node *memop,
                               ir_node *op1, ir_node *op2, ir_mode *mode,
-                              op_pin_state state);
+                              int pinned);
 
 /** Constructor for a remainderless Div node.
  *
@@ -666,11 +412,11 @@ FIRM_API ir_node *new_r_DivRL(ir_node *block, ir_node *memop,
  * @param   *op1   The first operand.
  * @param   *op2   The second operand.
  * @param   *mode  The mode of the result.
- * @param   state  The pinned state.
+ * @param   pinned Whether the node is pinned in its block.
  */
 FIRM_API ir_node *new_d_DivRL(dbg_info *db, ir_node *memop,
                               ir_node *op1, ir_node *op2, ir_mode *mode,
-                              op_pin_state state);
+                              int pinned);
 
 /** Constructor for a remainderless Div node.
  *
@@ -680,10 +426,10 @@ FIRM_API ir_node *new_d_DivRL(dbg_info *db, ir_node *memop,
  * @param   *op1   The first operand.
  * @param   *op2   The second operand.
  * @param   *mode  The mode of the result.
- * @param   state  The pinned state.
+ * @param   pinned Whether the node is pinned in its block.
  */
 FIRM_API ir_node *new_DivRL(ir_node *memop, ir_node *op1, ir_node *op2,
-                            ir_mode *mode, op_pin_state state);
+                            ir_mode *mode, int pinned);
 
 /** @} */
 
@@ -928,11 +674,12 @@ FIRM_API void keep_alive(ir_node *ka);
 /** Puts the graph into state "phase_high" */
 FIRM_API void irg_finalize_cons(ir_graph *irg);
 
-/** Puts the program and all graphs into state phase_high.
- *
- * This also remarks, the construction of types is finished,
- * e.g., that no more subtypes will be added.  */
-FIRM_API void irp_finalize_cons(void);
+/**
+ * If firm is built in debug mode, verify that a newly created node is fine.
+ * The normal node constructors already call this function, you only need to
+ * call this yourself if you create new node constructors on your own.
+ */
+FIRM_API void verify_new_node(ir_node *node);
 
 /**
  * Register a new callback for the case that the value of an uninitialized
