@@ -1,20 +1,6 @@
 /*
- * Copyright (C) 1995-2008 University of Karlsruhe.  All right reserved.
- *
  * This file is part of libFirm.
- *
- * This file may be distributed and/or modified under the terms of the
- * GNU General Public License version 2 as published by the Free Software
- * Foundation and appearing in the file LICENSE.GPL included in the
- * packaging of this file.
- *
- * Licensees holding valid libFirm Professional Edition licenses may use
- * this file in accordance with the libFirm Commercial License.
- * Agreement provided with the Software.
- *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE.
+ * Copyright (C) 2012 University of Karlsruhe.
  */
 
 /**
@@ -89,8 +75,8 @@
  *   value/mode pair.
  *
  * @note
- *   If the string is not representable in the given mode an assertion is
- *   thrown in assert build.
+ *   Behaviour is undefined if the number in @p str is malformed. Debug builds
+ *   should fail assertions in these cases.
  *
  * @sa
  *   irmode.h for predefined modes
@@ -103,13 +89,13 @@ FIRM_API ir_tarval *new_tarval_from_str(const char *str, size_t len,
 /**
  * Construct a new tarval from a given string.
  *
- * @param str   The string representing the target value
- * @param len   The length of the string
- * @param sign  is -1 or 1 depending on the numbers sign
- * @param base  number system base.
- *              binary(2), octal(8), decimal(10) and hexadecimal(16) numbers
- *              are supported.
- * @param mode  The mode requested for the result tarval
+ * @param str      The string representing the target value
+ * @param len      The length of the string
+ * @param negative != 0 if number should be negative
+ * @param base     number system base.
+ *                 binary(2), octal(8), decimal(10) and hexadecimal(16) numbers
+ *                 are supported.
+ * @param mode     The mode requested for the result tarval
  *
  * @return
  *   A tarval with the given mode. If overflow settings are set to
@@ -118,14 +104,16 @@ FIRM_API ir_tarval *new_tarval_from_str(const char *str, size_t len,
  *   Returns bad if the number couldn't successfully be parsed.
  */
 FIRM_API ir_tarval *new_integer_tarval_from_str(const char *str, size_t len,
-                                                char sign, unsigned char base,
+                                                int negative,
+                                                unsigned char base,
                                                 ir_mode *mode);
 
 /**
  * Constructor function for new tarvals
  *
  * @param l     The long representing the value
- * @param mode  The mode requested for the result tarval
+ * @param mode  The mode requested for the result tarval must be a
+ *              twos_complement mode.
  *
  * This function creates a new tarval representing the value represented
  * by a long integer. If a tarval representing this value already exists,
@@ -150,26 +138,48 @@ FIRM_API ir_tarval *new_integer_tarval_from_str(const char *str, size_t len,
  */
 FIRM_API ir_tarval *new_tarval_from_long(long l, ir_mode *mode);
 
-/** Returns value as long if possible.
+/**
+ * Construct a new tarval from a sequence of bytes. The bytes are interpreted
+ * in a "little endian" fashion which means less significant bytes come first.
  *
- * This returns a long int with the value represented value, or
- * gibberish, depending on the size of long int and the size of the
- * stored value. It works for e.g. 1 as mode_Ls, but might not work for
- * get_mode_max(mode_Ls).
- * This will overflow silently, so use only if you know what
- * you are doing! (better check with tarval_is_long()...)
- * Works only for int modes, even not for character modes!
+ * @param buf  pointer to a buffer holding at least
+ *             get_ir_mode_size_bytes(mode) bytes.
+ * @param mode mode for the resulting tarval
+ * @return A newly created (or cached) tarval representing the value.
  */
-FIRM_API long get_tarval_long(ir_tarval *tv);
+FIRM_API ir_tarval *new_tarval_from_bytes(unsigned char const *buf,
+                                          ir_mode *mode);
 
 /**
- * This validates if get_tarval_long() will return a satisfying
- * result. I.e. if tv is an int_number and between min, max
- * of long int (signed!)
+ * Construct a new floating point quiet NaN value.
+ * @param mode       floating point mode for the resulting value
+ * @param signaling  if != 0 produces a signaling NaN else a quiet one.
+ * @param payload    if != NULL puts the integer tarval into the mantissa.
+ */
+FIRM_API ir_tarval *new_tarval_nan(ir_mode *mode, int signaling,
+                                   ir_tarval *payload);
+
+/**
+ * Write tarval to a sequence of bytes. The value is written in a
+ * "little endian" fashion which means the less significant bytes come first.
+ */
+FIRM_API void tarval_to_bytes(unsigned char *buffer, ir_tarval *tv);
+
+/**
+ * Returns value as long if possible.
+ */
+FIRM_API long get_tarval_long(const ir_tarval *tv);
+
+/**
+ * This validates if get_tarval_long() will return something sensible.
+ * This is the case if the value is a two_complement (integer/reference) mode
+ * and converting it to a mode equivalent to "long" would not result in
+ * information loss. So ULONGMAX in an unsigned mode is fine, ULONG_MAX in a
+ * signed mode not.
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_long(ir_tarval *tv);
+FIRM_API int tarval_is_long(const ir_tarval *tv);
 
 /**
  * Constructor function for new tarvals.
@@ -214,12 +224,12 @@ FIRM_API ir_tarval *new_tarval_from_long_double(long double d, ir_mode *mode);
  *
  * @param tv    the tarval
  */
-FIRM_API double get_tarval_double(ir_tarval *tv);
+FIRM_API double get_tarval_double(const ir_tarval *tv);
 
 /**
  * same as get_tarval_double but returns a long double value
  */
-FIRM_API long double get_tarval_long_double(ir_tarval *tv);
+FIRM_API long double get_tarval_long_double(const ir_tarval *tv);
 
 /**
  * This validates if tarval_to_double() will return a satisfying
@@ -228,7 +238,7 @@ FIRM_API long double get_tarval_long_double(ir_tarval *tv);
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_double(ir_tarval *tv);
+FIRM_API int tarval_is_double(const ir_tarval *tv);
 
 /**
  * Returns the mode of the tarval.
@@ -242,49 +252,45 @@ FIRM_API ir_mode *get_tarval_mode(const ir_tarval *tv);
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_negative(ir_tarval *tv);
+FIRM_API int tarval_is_negative(const ir_tarval *tv);
 
 /**
  * Returns 1 if tv is null
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_null(ir_tarval *tv);
+FIRM_API int tarval_is_null(const ir_tarval *tv);
 
 /**
  * Returns 1 if tv is the "one"
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_one(ir_tarval *tv);
+FIRM_API int tarval_is_one(const ir_tarval *tv);
 
 /**
- * Returns 1 if tv is the "minus one"
- *
- * @param tv    the tarval
+ * returns non-zero if all bits in the tarval are set.
+ * This means the value is -1 for signed modes with irma_twos_complement.
  */
-FIRM_API int tarval_is_minus_one(ir_tarval *tv);
+FIRM_API int tarval_is_all_one(const ir_tarval *tv);
 
 /**
- * returns non-zero if all bits in the tarval are set
- */
-FIRM_API int tarval_is_all_one(ir_tarval *tv);
-
-/**
- * Returns non-zero if the tarval is a constant (ie. NOT
+ * Returns non-zero if the tarval is a constant (i.e. NOT
  * a reserved tarval like bad, undef, reachable etc.)
  */
-FIRM_API int tarval_is_constant(ir_tarval *tv);
+FIRM_API int tarval_is_constant(const ir_tarval *tv);
 
-/** The 'bad' tarval. */
-FIRM_API ir_tarval *tarval_bad;
+/** The 'bad' tarval. Representing "no value", do not confuse this with
+ * tarval_unknown. */
+FIRM_API ir_tarval *const tarval_bad;
 /** Returns the 'bad' tarval. */
 FIRM_API ir_tarval *get_tarval_bad(void);
 
-/** The 'undefined' tarval. */
-FIRM_API ir_tarval *tarval_undefined;
-/** Returns the 'undefined' tarval. */
-FIRM_API ir_tarval *get_tarval_undefined(void);
+/** The 'unknown' tarval. Representing an unknown (but legal) value, do
+ * not confuse this with tarval_bad. */
+FIRM_API ir_tarval *const tarval_unknown;
+/** Returns the 'unknown' tarval. */
+FIRM_API ir_tarval *get_tarval_unknown(void);
 
 /** The mode_b tarval 'false'. */
 FIRM_API ir_tarval *tarval_b_false;
@@ -296,74 +302,15 @@ FIRM_API ir_tarval *tarval_b_true;
 /** Returns the mode_b tarval 'true'. */
 FIRM_API ir_tarval *get_tarval_b_true(void);
 
-/** The mode_X tarval 'unreachable'. */
-FIRM_API ir_tarval *tarval_unreachable;
-/** Returns the mode_X tarval 'unreachable'. */
-FIRM_API ir_tarval *get_tarval_unreachable(void);
-
-/** The mode_X tarval 'reachable'. */
-FIRM_API ir_tarval *tarval_reachable;
-/** Returns the mode_X tarval 'reachable'. */
-FIRM_API ir_tarval *get_tarval_reachable(void);
-
-/** The 'top' tarval. This is just another name for the 'undefined' tarval. */
-#define tarval_top          tarval_undefined
-/** Returns the 'top' tarval. */
-#define get_tarval_top()    get_tarval_undefined()
-
-/** The 'bottom' tarval. This is just another name for the 'bad' tarval. */
-#define tarval_bottom       tarval_bad
-/** Returns the 'bottom' tarval. */
-#define get_tarval_bottom() get_tarval_bad()
-
-/** Returns the maximum value of a given mode. */
-FIRM_API ir_tarval *get_tarval_max(ir_mode *mode);
-
-/** Returns the minimum value of a given mode. */
-FIRM_API ir_tarval *get_tarval_min(ir_mode *mode);
-
-/** Returns the 0 value (additive neutral) of a given mode.
-    For reference modes, the NULL value is returned (old tarval_P_void) */
-FIRM_API ir_tarval *get_tarval_null(ir_mode *mode);
-
-/** Returns the 1 value (multiplicative neutral) of a given mode. */
-FIRM_API ir_tarval *get_tarval_one(ir_mode *mode);
-
-/** Returns the -1 value (multiplicative neutral) of a given mode.
- *  Returns tarval bad for unsigned modes */
-FIRM_API ir_tarval *get_tarval_minus_one(ir_mode *mode);
-
-/** returns the value where all bits are 1 of a given mode.
- * returns tarval_bad for float modes */
-FIRM_API ir_tarval *get_tarval_all_one(ir_mode *mode);
-
-/** Returns quite nan for float_number modes. */
-FIRM_API ir_tarval *get_tarval_nan(ir_mode *mode);
-
-/** Returns +inf for float_number modes. */
-FIRM_API ir_tarval *get_tarval_plus_inf(ir_mode *mode);
-
-/** Returns -inf for float_number modes. */
-FIRM_API ir_tarval *get_tarval_minus_inf(ir_mode *mode);
-
-/** Modes for handling integer overflows. */
-typedef enum tarval_int_overflow_mode_t {
-	TV_OVERFLOW_BAD,      /**< tarval module will return tarval_bad if a overflow occurs */
-	TV_OVERFLOW_WRAP,     /**< tarval module will overflow will be ignored, wrap around occurs */
-	TV_OVERFLOW_SATURATE  /**< tarval module will saturate the overflow */
-} tarval_int_overflow_mode_t;
+/**
+ * Sets whether values should wrap on overflow or return the bad value.
+ */
+FIRM_API void tarval_set_wrap_on_overflow(int wrap_on_overflow);
 
 /**
- * Sets the overflow mode for integer operations.
- *
- * @param ov_mode  one of the overflow modes
+ * Returns 0 if operations return bad on overflow, != 0 if they wrap around.
  */
-FIRM_API void tarval_set_integer_overflow_mode(tarval_int_overflow_mode_t ov_mode);
-
-/**
- * Returns the overflow mode for integer operations.
- */
-FIRM_API tarval_int_overflow_mode_t tarval_get_integer_overflow_mode(void);
+FIRM_API int tarval_get_wrap_on_overflow(void);
 
 /**
  * Compares two tarvals
@@ -376,7 +323,7 @@ FIRM_API tarval_int_overflow_mode_t tarval_get_integer_overflow_mode(void);
  * @param a   the first tarval to be compared
  * @param b   the second tarval to be compared
  */
-FIRM_API ir_relation tarval_cmp(ir_tarval *a, ir_tarval *b);
+FIRM_API ir_relation tarval_cmp(const ir_tarval *a, const ir_tarval *b);
 
 /**
  * Converts a tarval to another mode.
@@ -394,7 +341,7 @@ FIRM_API ir_relation tarval_cmp(ir_tarval *a, ir_tarval *b);
  *   constructed and returned
  *
  * @note
- *    Illegal convertions will trigger a panic
+ *    Illegal conversions will trigger a panic
  *
  * @sa
  *    FIRM documentation for conversion rules
@@ -402,6 +349,13 @@ FIRM_API ir_relation tarval_cmp(ir_tarval *a, ir_tarval *b);
  */
 FIRM_API ir_tarval *tarval_convert_to(ir_tarval *src, ir_mode *mode);
 
+/**
+ * Converts a tarval to another mode by reinterpreting the contained
+ * bits. This is only allowed if the source and destination mode have the
+ * same number of bits.
+ */
+FIRM_API ir_tarval *tarval_bitcast(ir_tarval *src, ir_mode *mode);
+
 /*
  * These function implement basic computations representable as opcodes
  * in FIRM nodes.
@@ -460,11 +414,10 @@ FIRM_API ir_tarval *tarval_add(ir_tarval *a, ir_tarval *b);
  *
  * @param a         the first tarval
  * @param b         the second tarval
- * @param dst_mode  the mode of the result, needed for mode_P - mode_P, else NULL
  *
  * @return a - b or tarval_bad
  */
-FIRM_API ir_tarval *tarval_sub(ir_tarval *a, ir_tarval *b, ir_mode *dst_mode);
+FIRM_API ir_tarval *tarval_sub(ir_tarval *a, ir_tarval *b);
 
 /**
  * Multiplication of tarvals.
@@ -546,6 +499,16 @@ FIRM_API ir_tarval *tarval_andnot(ir_tarval *a, ir_tarval *b);
  */
 FIRM_API ir_tarval *tarval_or(ir_tarval *a, ir_tarval *b);
 
+/**
+ * Bitwise or not of two integer tarvals.
+ *
+ * @param a  the first tarval
+ * @param b  the second tarval
+ *
+ * @return a | ~b or tarval_bad
+ */
+FIRM_API ir_tarval *tarval_ornot(ir_tarval *a, ir_tarval *b);
+
 /**
  * Bitwise exclusive or of two integer tarvals.
  *
@@ -604,105 +567,10 @@ FIRM_API ir_tarval *tarval_shrs(ir_tarval *a, ir_tarval *b);
  */
 FIRM_API ir_tarval *tarval_shrs_unsigned(ir_tarval *a, unsigned b);
 
-/**
- * Rotation to left.
- *
- * @param a  the first tarval
- * @param b  the second tarval
- *
- * @return a \<\<L\>\> b or tarval_bad
- */
-FIRM_API ir_tarval *tarval_rotl(ir_tarval *a, ir_tarval *b);
-
-/**
- * Returns the carry flag of the last operation.
- */
-FIRM_API int tarval_carry(void);
-
-/**
- * The output mode for tarval values.
- *
- * Some modes allow more that one representation, for instance integers
- * can be represented hex or decimal. Of course it would be enough to have
- * one and let every backend convert it into the 'right' one.
- * However, we can do this in the tarval much simpler...
- */
-typedef enum {
-	TVO_NATIVE,  /**< the default output mode, depends on the mode */
-	TVO_HEX,     /**< use hex representation, always possible */
-	TVO_DECIMAL, /**< use decimal representation */
-	TVO_OCTAL,   /**< use octal representation */
-	TVO_BINARY,  /**< use binary representation */
-	TVO_FLOAT,   /**< use floating point representation (i.e 1.342e-2)*/
-	TVO_HEXFLOAT /**< use hexadecimal floating point representation
-	                  (i.e 0x1.ea32p-12)*/
-} tv_output_mode;
-
-/**
- * This structure contains helper information to format the output
- * of a tarval of a mode.
- */
-typedef struct tarval_mode_info {
-	tv_output_mode mode_output;  /**< if != TVO_NATIVE select a special mode */
-	const char *mode_prefix;     /**< if set, this prefix will be printed
-	                                  before a value of this mode */
-	const char *mode_suffix;     /**< if set, this suffix will be printed
-	                                  after a value of this mode */
-} tarval_mode_info;
-
-/**
- * Specify the output options of one mode.
- *
- * This functions stores the mode info, so DO NOT DESTROY it.
- *
- * @param mode      a ir_mode that should be associated
- * @param modeinfo  the output format info
- *
- * @return zero on success.
- */
-FIRM_API int set_tarval_mode_output_option(ir_mode *mode,
-                                           const tarval_mode_info *modeinfo);
-
-/**
- * Returns the output options of one mode.
- *
- * This functions returns the mode info of a given mode.
- *
- * @param mode      a ir_mode that should be associated
- *
- * @return the output option
- */
-FIRM_API const tarval_mode_info *get_tarval_mode_output_option(ir_mode *mode);
-
-/**
- * Returns Bit representation of a tarval value, as string of '0' and '1'
- *
- * @param tv   The tarval
- *
- * This function returns a printable bit representation of any value
- * stored as tarval. This representation is a null terminated C string.
- *
- * @return
- *   As usual in C a pointer to a char is returned. The length of the
- *   returned string if fixed, just read as many chars as the mode defines
- *   as size.
- *
- * @note
- *   The string is allocated using malloc() and is free()ed on the next call
- *   of this function.
- *   The string consists of the ASCII characters '0' and '1' and is
- *   null terminated
- *
- * @sa
- *    irmode.h for the definition of the ir_mode struct
- *    the size member of aforementioned struct
- */
-FIRM_API char *get_tarval_bitpattern(ir_tarval *tv);
-
 /**
  * Returns the bitpattern of the bytes_ofs byte.
  *
- * This function succeeds even if the mode of the tarval uses lesser bits
+ * This function succeeds even if the mode of the tarval uses fewer bits
  * than requested, in that case the bitpattern is filled with zero bits.
  *
  * To query a 32bit value the following code can be used:
@@ -725,15 +593,7 @@ FIRM_API char *get_tarval_bitpattern(ir_tarval *tv);
  *   The result of this function is undefined if the mode is neither integer
  *   nor float.
  */
-FIRM_API unsigned char get_tarval_sub_bits(ir_tarval *tv, unsigned byte_ofs);
-
-/**
- * Returns non-zero if a given (integer) tarval has only one single bit
- * set.
- *
- * @param tv    the tarval
- */
-FIRM_API int tarval_is_single_bit(ir_tarval *tv);
+FIRM_API unsigned char get_tarval_sub_bits(const ir_tarval *tv, unsigned byte_ofs);
 
 /**
  * Returns the number of set bits in a given (integer) tarval.
@@ -742,7 +602,7 @@ FIRM_API int tarval_is_single_bit(ir_tarval *tv);
  *
  * @return number of set bits or -1 on error
  */
-FIRM_API int get_tarval_popcount(ir_tarval *tv);
+FIRM_API int get_tarval_popcount(const ir_tarval *tv);
 
 /**
  * Returns the number of the lowest set bit in a given (integer) tarval.
@@ -751,23 +611,16 @@ FIRM_API int get_tarval_popcount(ir_tarval *tv);
  *
  * @return number of lowest set bit or -1 on error
  */
-FIRM_API int get_tarval_lowest_bit(ir_tarval *tv);
+FIRM_API int get_tarval_lowest_bit(const ir_tarval *tv);
 
 /**
- * Output a tarval to a string buffer.
- *
- * @param buf     the output buffer
- * @param buflen  the length of the buffer
- * @param tv      the tarval
- */
-FIRM_API int tarval_snprintf(char *buf, size_t buflen, ir_tarval *tv);
-
-/**
- * Output a tarval to stdio.
+ * Return the number of the highest set bit in a given (integer) tarval.
  *
  * @param tv    the tarval
+ *
+ * @return number of highest set bit or -1 on error
  */
-FIRM_API int tarval_printf(ir_tarval *tv);
+FIRM_API int get_tarval_highest_bit(const ir_tarval *tv);
 
 /**
  * Returns non-zero if the mantissa of a floating point tarval is zero
@@ -775,7 +628,7 @@ FIRM_API int tarval_printf(ir_tarval *tv);
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_zero_mantissa(ir_tarval *tv);
+FIRM_API int tarval_zero_mantissa(const ir_tarval *tv);
 
 /**
  * Returns the exponent of a floating point IEEE-754
@@ -783,7 +636,7 @@ FIRM_API int tarval_zero_mantissa(ir_tarval *tv);
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_get_exponent(ir_tarval *tv);
+FIRM_API int tarval_get_exponent(const ir_tarval *tv);
 
 /**
  * Check if the tarval can be converted to the given mode without
@@ -792,7 +645,7 @@ FIRM_API int tarval_get_exponent(ir_tarval *tv);
  * @param tv    the tarval
  * @param mode  the mode to convert to
  */
-FIRM_API int tarval_ieee754_can_conv_lossless(ir_tarval *tv, ir_mode *mode);
+FIRM_API int tarval_ieee754_can_conv_lossless(const ir_tarval *tv, const ir_mode *mode);
 
 /**
  * Returns non-zero if the result of the last IEEE-754 operation was exact.
@@ -800,50 +653,32 @@ FIRM_API int tarval_ieee754_can_conv_lossless(ir_tarval *tv, ir_mode *mode);
 FIRM_API unsigned tarval_ieee754_get_exact(void);
 
 /**
- * Enable/Disable floating point constant folding.
- */
-FIRM_API void tarval_enable_fp_ops(int enable);
-
-/** returns 0/1 if floating point folding is enable/disabled */
-FIRM_API int tarval_fp_ops_enabled(void);
-
-/**
- * Check if its the a floating point NaN.
+ * Check if @p tv is a floating point NaN.
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_NaN(ir_tarval *tv);
+FIRM_API int tarval_is_nan(const ir_tarval *tv);
 
 /**
- * Check if its the a floating point +inf.
+ * Check if @p tv is a floating point quiet NaN.
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_plus_inf(ir_tarval *tv);
+FIRM_API int tarval_is_quiet_nan(const ir_tarval *tv);
 
 /**
- * Check if its the a floating point -inf.
+ * Check if @p tv is a floating point signaling NaN.
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_minus_inf(ir_tarval *tv);
+FIRM_API int tarval_is_signaling_nan(const ir_tarval *tv);
 
 /**
  * Check if the tarval represents a finite value, ie neither NaN nor inf.
  *
  * @param tv    the tarval
  */
-FIRM_API int tarval_is_finite(ir_tarval *tv);
-
-/**
- *   Checks whether a pointer points to a tarval.
- *
- *   @param thing     an arbitrary pointer
- *
- *   @return
- *       true if the thing is a tarval, else false
- */
-FIRM_API int is_tarval(const void *thing);
+FIRM_API int tarval_is_finite(const ir_tarval *tv);
 
 /** @} */
 
